(module spico-lang mzscheme
  (require (lib "tool.ss" "drscheme")
	   (lib "mred.ss" "mred")
           (lib "framework.ss" "framework")
           (lib "unitsig.ss")
	         (lib "contract.ss")
           (lib "class.ss")
	   (lib "turtles.ss" "graphics")
           "spico-strings.ss"
           "lexer.ss"
           "spico-eval.ss")

  (provide tool@)
 
  ;; This is the main language tool. A tool consists of 2 phases.
  ;; In the second phase we build the language, and add the language
  ;; to the configuration.
  (define tool@
    (unit/sig drscheme:tool-exports^
      (import drscheme:tool^)

      (define test 'dummy)

      (define (phase1)
        
        (drscheme:get/extend:extend-unit-frame 
          (lambda (super%)
            (class super%
                   (super-new)
                   (define panel 
                     (send this get-info-panel))

                   (define bitmap (make-object bitmap% "spico-icon.png" 'png #f))
                   (define canvas 
                     (instantiate canvas% () 
                                  (parent panel)
                                  (min-width 32)
                                  (min-height 32)
                                  (stretchable-width #f)
                                  (stretchable-height #f)
                                  (paint-callback (lambda (canvas dc)
                                                    (send dc draw-bitmap bitmap 0 0)))))
                                                    
                   (define dc (send canvas get-dc))
                   (set! test dc)

                   ;(define/public (show-pico)
                   ;               (send dc draw-bitmap bitmap 0 0))
                   ;(define/public (unshow-pico)
                   ;               (send dc clear))

                   (send dc set-background (make-object color% 255 0 0))
                   (send dc clear)
                   (send dc draw-bitmap bitmap 0 0)))))
                   
      
      (define (phase2)
        (drscheme:language-configuration:add-language (make-object lang%)))

      ;; short-sym->pref-name : symbol -> symbol
      ;; returns the preference name for the color prefs
      (define (short-sym->pref-name sym) (string->symbol (short-sym->style-name sym)))

      ;; short-sym->style-name : symbol->string
      ;; converts the short name (from the table above) into a name in the editor list
      ;; (they are added in by `color-prefs:register-color-pref', called below)
      (define (short-sym->style-name sym) (format "~a:syntax-coloring:scheme:~a" spico-name sym))

      ;; extend-preferences-panel : vertical-panel -> void
      ;; adds in the configuration for the sPico colors to the prefs panel
      (define (extend-preferences-panel parent)
        (for-each
         (lambda (line)
           (let ([sym (car line)])
             (color-prefs:build-color-selection-panel
              parent
              (short-sym->pref-name sym)
              (short-sym->style-name sym)
              (format "~a" sym))))
         color-prefs-table))

      ;; Standard spico-syntax-coloring
      (define color-prefs-table
        `((keyword    ,(make-object color% "blue")         "Keyword")
          (string     ,(make-object color% "forestgreen")  "String")
          (literal    ,(make-object color% "forestgreen")  "Literal")
          (comment    ,(make-object color% 194 116 31)     "Comment")
          (error      ,(make-object color% "red")          "Error")
          (identifier ,(make-object color% 38 38 128)      "Identifier")
          (seperator  ,(make-object color% 194 116 31)     "Seperator")
          (default    ,(make-object color% "black")        "Default")))

      ;; Function for the interaction-window. Checks if text is ready
      ;; to be sent to the engine (all parens etc are correctly closed)
      ;; when more parens are closed then opened, we also send text.
      (define repl-submit
        (lambda (text prompt-position)
          (let [(open-brackets 0) ;; [ ]
                (open-curlies 0)  ;; { }
                (open-parens 0)   ;; ( )
                (string1? #f)     ;; ' '
                (string2? #f)     ;; " "
                (comment? #f)]    ;; ` `]
                (let loop [(current-pos (+ prompt-position 1))
                           (current-char (send text get-character prompt-position))]
                         ;; Submit if more parens,brackets or curlies are closed then opened
                  (cond ((or (< open-brackets 0)
                             (< open-curlies 0)
                             (< open-parens 0)) #t)
                        ;; If all text is read, text must be submitted if everything is correctly closed
                        ((eq? current-char #\nul) (and (= open-brackets 0)
                                                       (= open-curlies 0)
                                                       (= open-parens 0)
                                                       (not string1?)
                                                       (not string2?)
                                                       (not comment?)))
                        ;; If not read yet, read text.
                        (else
                                ;; Ignore everything except for closing-sign
                          (cond (string1? (when (eq? current-char #\')
                                            (set! string1? #f)))
                                (string2? (when (eq? current-char #\")
                                            (set! string2? #f)))
                                (comment? (when (eq? current-char #\`)
                                            (set! comment? #f)))
                                ;; If not in string or comment, react to special chars.
                                (else (case current-char
                                        ((#\') (set! string1? #t))
                                        ((#\") (set! string2? #t))
                                        ((#\`) (set! comment? #t))
                                        ((#\() (set! open-parens (+ open-parens 1)))
                                        ((#\{) (set! open-curlies (+ open-curlies 1)))
                                        ((#\[) (set! open-brackets (+ open-brackets 1)))
                                        ((#\)) (set! open-parens (- open-parens 1)))
                                        ((#\}) (set! open-curlies (- open-curlies 1)))
                                        ((#\]) (set! open-brackets (- open-brackets 1))))))
                          ;; Call loop with next character.
                          (loop (+ current-pos 1)
                                (send text get-character current-pos))))))))

      ;; Check if given language is the Pico-language (pretty ugly :)
      (define (matches-language l)
        (and l (pair? l) (pair? (cdr l))
             (equal? (cadr l) pico-type)))

      ;; The sPico mode-surrogate
      (define mode-surrogate
         (new color:text-mode%
             (matches (list (list '|{| '|}|)
                            (list '|(| '|)|)
                            (list '|[| '|]|)))
             (get-token get-syntax-token)
             (token-sym->style short-sym->style-name)))

      ;; Adding the sPico-mode to the drscheme-modes
      (drscheme:modes:add-mode (string-append spico-name " Mode")
                               mode-surrogate
                               repl-submit
                               matches-language)
      
      ;; Adding sPico-subdir to preferences-panel
      (color-prefs:add-to-preferences-panel spico-name extend-preferences-panel)

      ;; Adding the color-prefs to the preference-table
      (for-each (lambda (line)
                  (let ([sym (car line)]
                        [color (cadr line)])
                    (color-prefs:register-color-pref (short-sym->pref-name sym)
                                                     (short-sym->style-name sym)
                                                     color)))
                color-prefs-table)
      
      ;; Semaphore to make sure only one session is using
      ;; the pico-engine at a time
      (define semaphore (make-semaphore 1))

      ;; The following specifies the sPico-language object.
      ;; the most important part is the front-end method,
      ;; which read-eval-prints the input.
      (define lang%
        (class* object% (drscheme:language:language<%>)
          (define/public (config-panel parent)
            (case-lambda
              [() null]
              [(x) (void)]))
          (define/public (default-settings) null)
          (define/public (default-settings? x) #t)
          ;; Main function that calls pico-eval.
          (define/private (front-end input settings)
            ;; Save done? to loop once more in the lambda. First time to
            ;; return value, second time to end pico-eval
            (send test set-background (make-object color% 255 0 0))
            (let* [(done? #f)
                   (rep-text (drscheme:language:text/pos-text input))
                   (posstart (drscheme:language:text/pos-start input))
                   (posend (drscheme:language:text/pos-end input))
                   (str (send rep-text get-text posstart posend))]

              ;; talk-to-pico talks to the pico-eval through scheme-symbols.
              
              ;; Given + argument:
              ;;
              ;; - spico-start: start evaluating the given string
              ;; - spico-continue: continue evaluation (ignore dummy-symbol)
              ;; - spico-accept: accept the given string (when pressed the eof-box, give empty-string)

              ;; Result:
              ;;
              ;; - spico-wait: Redo the talk-to-pico with spico-continue,
              ;;               because of this step other processes will get the semaphore
              ;; - spico-error: an error has occurred, error will be shown from the inside,
              ;;                and a pico-mark will follow
              ;; - a scheme-pair: containing mark-positions for error.
              
              (define (talk-to-pico type value)
                    ;; Wait for the semaphore to eval code.
                    (semaphore-wait semaphore)
                    ;; Eval code in the pico-engine
                    (let [(result (pico-eval type value))]
                      ;; Release semaphore after evalling.
                      (semaphore-post semaphore)
                      ;; Case result using the possible results specified above
                      (case result
                        ((spico-wait)
                         (talk-to-pico 'spico-continue 'dummy))
                        ((spico-accept)
                         (let [(accept (read-line))]
                           (if (eof-object? accept)
                             (talk-to-pico 'spico-accept "")
                             (talk-to-pico 'spico-accept accept))))
                        ;; #` and #, is syntax-quote stuff -> Create a void syntax-object
                        ((spico-error)
                         #`#,(void))
                        (else 
                          (if (pair? result)
                            (begin
                              ;; Wait for window to be ready to highlight error,
                              ;; not doing so will cause the error-highlight to appear
                              ;; and directly disappear.
                              (send (drscheme:rep:current-rep) wait-for-io-to-complete)
                              (send (drscheme:rep:current-rep) highlight-error rep-text 
                                    (+ (car result) posstart)
                                    (+ (cdr result) posstart))
                              ;; The result of an error is an eof result.
                              eof)
                            ;; If none of the previous has occured, result is the full result.
                            #`#,result)))))

              ;; This contains a hack to receive the value and return, then stop the program.
              ;; First run the result is returned, scheme will automatically restart the loop,
              ;; then we return eof.
              
              (lambda ()
                (if (or done? (equal? str ""))
                    eof
                    (let ((result (talk-to-pico 'spico-start str)))
                      (set! done? #t)
                      result)))))
          
          (define/public (front-end/complete-program input settings teachpack-cache) (front-end input settings))
          (define/public (front-end/interaction input settings teachpack-cache) (front-end input settings))
          (define/public (get-style-delta) #f)
          (define/public (get-language-position) (list "sPico" pico-type))
          (define/public (order-manuals x) (values '("pico" "tour" "drscheme" "help") #f))
          (define/public (get-language-name) spico-name)
          (define/public (get-language-url) spico-url)
          (define/public (get-language-numbers) (list 1 pico-version))
          (define/public (marshall-settings x) x)
          (define/public (on-execute settings run-in-user-thread)
                         'ok)
          (define/public (render-value value settings port port-write) (display value port))
          (define/public (render-value/format value settings port port-write width) (display value port))
          (define/public (unmarshall-settings x) x)
          (define/public (create-executable settings parent src-file teachpacks) #f)
          (define/public (get-one-line-summary) "A Pico GUI that really rocks! (by Kim Gybels and Toon Verwaest)")
          (super-instantiate ()))))))

